/**
 * Demo of the Mandelbrot fractal renderer
 * 
 * Copyright (c) 2008 Tom Beddard
 * http://www.subblue.com
 * 
 * Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
 */

package
{
	import flash.display.*;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.utils.*;
	import flash.geom.Rectangle;
	import flash.geom.Point;
	import flash.ui.Keyboard;
	import com.subblue.fractal.*;
	import com.subblue.fractal.mappings.*;
	
	public class Main extends Sprite
	{
		private var fractal:Fractal;
		private var mappingIndex:int = 0;
		private var panAmount:int = 50;
		private var _maxIteractions:int = 250;
		
		// Note: the c attribute is here so that the class will be found by 
		// the getDefinitionByName function.
		private var mappingModes:Array = [
		 {mode:'SmoothMapping', 	   bailout:64,  bailoutType:'standard', c:SmoothMapping},
		 {mode:'ShelfMapping',  	   bailout:16,  bailoutType:'standard', c:ShelfMapping},
		 {mode:'ShelfMapping',  	   bailout:16,  bailoutType:'spiky', 	c:ShelfMapping},
		 {mode:'ShelfMapping',  	   bailout:16,  bailoutType:'puffy', 	c:ShelfMapping},
		 {mode:'BinaryDecomposition1', bailout:128, bailoutType:'standard', c:BinaryDecomposition1}
		 ];
		
		
		
		public function Main()
		{
			stage.showDefaultContextMenu = false;
			stage.scaleMode = StageScaleMode.NO_SCALE;
		    stage.align = StageAlign.TOP_LEFT;
			init();
		}
		
		
		/**
		 *	Set everything up.
		 *	Note: we check for the stageWidth because sometimes FF3 returns null if we try
		 *	to initialise too quickly.
		 */
		private function init(e:Event = null):void
		{
			if (stage.stageWidth)
			{
			    stage.addEventListener(Event.RESIZE, resizeListener);
				stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDownListener);
				
				// Create the fractal system
				var bmpd:BitmapData = new BitmapData(stage.stageWidth, stage.stageHeight, false, 0xFF000000);
				var fractalBitmap:FractalBitmap = new Mandelbrot(bmpd);
				fractal = new Fractal(fractalBitmap);
				addChild(fractal);
				setMapping();
			} else {
				setTimeout(init, 100);
			}
		}
		
		
		/**
		 *	Resize the fractal when the stage changes
		 */
		private function resizeListener(e:Event):void
		{
			if (e) fractal.resize(stage.stageWidth, stage.stageHeight);
		}
		
		
		/**
		 *	Key press listener
		 */
		private function keyDownListener(e:KeyboardEvent):void
		{
			trace(e.keyCode);
			
			switch (e.keyCode)
			{
				// Pan the fractal
				case 37: // Left arrow
				fractal.move(-panAmount, 0);
				break;
				
				case 38: // Up arrow
				fractal.move(0, -panAmount);
				break;
				
				case 39: // Right arrow
				fractal.move(panAmount, 0);
				break;
				
				case 40: // Down arrow
				fractal.move(0, panAmount);
				break;
				
				// a: toggle anti-aliasing
				case 65:
				fractal.activeLayer.sampling = fractal.activeLayer.sampling == 2 ? 1 : 2;
				fractal.activeLayer.render();
				break;
				
				// f: toggle fullscreen
				case 70:
				stage.displayState = stage.displayState == StageDisplayState.NORMAL ? StageDisplayState.FULL_SCREEN : StageDisplayState.NORMAL;
				fractal.render();
				break;
				
				// m: set mapping mode
				case 77:
				setMapping();
				break;
				
				// r: reset
				case 82:
				fractal.activeLayer.reset();
				break;
				
				// +: increase max iterations (increase detail)
				case 187:
				fractal.activeLayer.maxIterations += 200;
				fractal.render();
				break;
				
				// -: decrease max iterations (increase detail)
				case 189:
				fractal.activeLayer.maxIterations -= 200;
				fractal.render();
				break;
			}
		}
		
		
		/**
		 *	Set mapping mode
		 */
		private function setMapping():void
		{
			var m:Object = mappingModes[mappingIndex];
			var MappingReference:Class = getDefinitionByName("com.subblue.fractal.mappings." + m.mode) as Class;
			var mapping:Mapping = new MappingReference();
			
			fractal.activeLayer.mapping = mapping;
			fractal.activeLayer.bailout = m.bailout;
			fractal.activeLayer.bailoutType = m.bailoutType;
			
			mappingIndex++;
			if (mappingIndex == mappingModes.length) mappingIndex = 0;
			fractal.render();
		}
		
		
	}
}