/**
 * Fractal.as
 * 
 * Copyright (c) 2008 Tom Beddard
 * http://www.subblue.com
 * 
 * Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
 */

package com.subblue.fractal
{
	
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.geom.Matrix;
	import flash.display.*;
	import flash.events.*;
	import flash.text.TextField;
	import com.subblue.fractal.*;
	
	
	public class Fractal extends Sprite
	{
		
		public 	  var activeLayer:FractalBitmap;
		private   var _zoomFactor:Number = 2;	// zoom amount
		private   var pressedKey:int;
		private   var dragging:Boolean = false;
		private   var dragPoint:Point;
		private   var _status:TextField;
		
		
		/**
		 *	Setup the fractal system
		 */
		public function Fractal(fractalBitmap:FractalBitmap = null)
		{
			addLayer(fractalBitmap);
			addEventListener(Event.ADDED_TO_STAGE, initialiseEvents);
			fractalBitmap.addEventListener(FractalBitmap.RENDER_QUEUE_COMPLETE, renderCompleteListener);
		}
		
		
		/**
		 *	Add fractal layer
		 */
		public function addLayer(fractalBitmap:FractalBitmap = null):void
		{
			if (fractalBitmap)
			{
				/*fractalBitmap.blendMode = bm;*/
				addChild(fractalBitmap);
				activeLayer = fractalBitmap;
			}
		}
		
		
		/**
		 *	Remove fractal layer
		 */
		public function removeLayer(idx:int):void
		{
			removeChildAt(idx);
		}
		
		
		/**
		 *	Render the fractal layers
		 */
		public function render():void
		{
			activeLayer.render();
		}
		
		
		/**
		 *	Resize the fractal and re-render it
		 */
		public function resize(w:Number, h:Number):void
		{
			for (var i:int = 0; i < numChildren; i++)
			{
				FractalBitmap(getChildAt(i)).resize(w, h);
				FractalBitmap(getChildAt(i)).render();
			}
		}
		
		
		/**
		 *	Pan the fractal layers
		 */
		public function move(dx:int, dy:int):void
		{
			for (var i:int = 0; i < numChildren; i++)
				FractalBitmap(getChildAt(i)).pan(dx, dy);
		}
		
		
		/**
		 *	Zoom all fractal layers
		 */
		public function zoom(dz, mx:Number, my:Number):void
		{
			for (var i:int = 0; i < numChildren; i++)
				FractalBitmap(getChildAt(i)).zoom(dz, mx, my);
		}
		
		
		/**
		 *  Return the requested fractal layer
		 */
		public function layer(i:int = 0):FractalBitmap
		{
			return FractalBitmap(getChildAt(i));
		}
		
		
		/**
		 *	Return the number of fractal layers
		 */
		public function get numLayers():int{
			return numChildren;
		}
		
		
		/**
		 *	Setup events
		 */
		private function initialiseEvents(e:Event):void
		{
			addEventListener(MouseEvent.MOUSE_DOWN,  mouseDownListener);
			addEventListener(MouseEvent.MOUSE_MOVE,  mouseMoveListener);
			addEventListener(MouseEvent.MOUSE_UP, 	 mouseUpListener);
			stage.addEventListener(MouseEvent.MOUSE_WHEEL, mouseWheelListener);
		}
		
		
		/**
		 *	Mouse down listener
		 */
		private function mouseDownListener(e:MouseEvent):void
		{
			var i:int;
			
			if (!dragging && e.ctrlKey)
			{
				dragging = true;
				dragPoint = new Point(mouseX, mouseY);
			}
			else if (e.shiftKey)
			{
				zoom(1/_zoomFactor, mouseX, mouseY);
			}
			else
			{
				zoom(_zoomFactor, mouseX, mouseY);
			}
		}
		
		
		/**
		 * Now dragging the fractal
		 */
		private function mouseMoveListener(e:MouseEvent):void
		{
			if (!dragging) return;
			
			x = int(stage.mouseX - dragPoint.x);
			y = int(stage.mouseY - dragPoint.y);
			e.updateAfterEvent();
		}
		
		
		/**
		 *	Stop dragging the fractal and redraw it
		 */
		private function mouseUpListener(e:MouseEvent):void
		{
			if (!dragging) return;
			dragging = false;
			x = y = 0;
			move(dragPoint.x - stage.mouseX, dragPoint.y - stage.mouseY);
		}
		
		
		/**
		 * Zooms the Mandelbrot on mouse wheel movement
		 */
		private function mouseWheelListener(e:MouseEvent):void
		{
			trace("Mouse wheel");
			var dz:Number = e.delta > 0 ? _zoomFactor : 1/_zoomFactor;
			
			for (var i:int = 0; i < numChildren; i++)
				FractalBitmap(getChildAt(i)).zoom(dz, mouseX, mouseY);
		}
		
		/**
		 *	Render complete listener
		 */
		private function renderCompleteListener(e:Event):void
		{
			if (_status)
			{
				_status.text = 'Complete';
			}
		}
		
		
		////////////////////////////////////////////////////
		//
		// Getter and Setter methods
		//
		////////////////////////////////////////////////////
		
		
		/**
		 *	Get and set zoom factor
		 */
		public function get zoomFactor():Number { 
			return _zoomFactor; 
		}
		public function set zoomFactor(value:Number):void {
			if (value !== _zoomFactor) _zoomFactor = value;
		}
		
	}
}