/**
 * Mandelbrot.as
 * 
 * Copyright (c) 2008 Tom Beddard
 * http://www.subblue.com
 * 
 * Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
 */

package com.subblue.fractal
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import com.subblue.fractal.Mapping;

	public class Mandelbrot extends FractalBitmap
	{
		
		public static const X_ORIGIN:Number = -0.5;
		public static const Y_ORIGIN:Number = 0;
		public static const DEFAULT_MAGNIFICATION:Number = 1;
		
		
		/**
		 * Setup the Mandelbrot fractal
		 * 
		 * @param	b		BitmapData to draw into
		 */
		public function Mandelbrot(b:BitmapData = null, mapping:Mapping = null)
		{
			super(b, mapping);
		}
		
		
		/**
		 *  Mandelbrot formula
		 *	z' = z^2 + c
		 *	 where: z = a + bi
		 *		    z^2 = (a^2 - b^2) + (2ab)i
		 *	 and: c = x + yi
		 */
		override protected function formula(x:Number, y:Number):Object
		{
			var n:int = 0;
			var a:Number = x;
			var b:Number = y;
			var c:Number;
			var zz:Number, aa:Number, bb:Number, twoab:Number;
			
			// Check to see if we are in the main cardioid
			// if (256 * (x2 + y2)^2 - 96 * (x2 + y2) + 32 * x - 3 <= 0)
			
			while (n < _maxIterations) {
				aa = a * a;
				bb = b * b;
				
				// If the magnitude of z is greater than 2 (or a^2 + b^2 > 4) 
				// then the sequence will tend to infinity
				c = bailoutCondition(aa, bb);
				if (c > _bailout) 
				{
					zz = c;
					break;
				}
				
				twoab = 2.0 * a * b;
				a = aa - bb + x;
				b = twoab + y;
				n++;
			}
			
			return {n:n, a:a, b:b, zz:zz};
		}
		
		
		/**
		 *	Bailout conditions - simple tweaks for the bailout conditions
		 */
		override protected function bailoutCondition(aa:Number, bb:Number):Number
		{
			if (_bailoutType == 'spiky') {
				return (aa - bb);
			} else if (_bailoutType == 'puffy') {
				return (bb - aa);
			} else {
				return (aa + bb);
			}		
		}
	}
}