/**
 * SmoothMapping.as
 * 
 * Copyright (c) 2008 Tom Beddard
 * http://www.subblue.com
 * 
 * Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
 * 
 * References:
 * http://en.wikipedia.org/wiki/Mandelbrot_set#Continuous_.28smooth.29_coloring
 */


package com.subblue.fractal.mappings
{
	import com.subblue.fractal.Mapping;
	import com.subblue.fractal.ColorPalette;
	import com.subblue.utils.ColorUtil;

    public class SmoothMapping extends Mapping
    {
		private var log2:Number = Math.log(2);
			
		/**
		 *	Create a continuous mapping
		 */
		public function SmoothMapping(palette:Array = null, size:int = 0)
        {
			super(palette, size);
        }
		
		
		/**
		 *	Return the colour value for the required index.
		 *	This is the linear transfer
		 *	
		 *	@param		re		Real component
		 *	@param		im		Imaginary component
		 *	@param		zz		Magnitude squared
		 *	@param		n		Number of iterations taken
		 */
		override public function map(re:Number, im:Number, zz:Number, n:int):uint
		{
			var p:Number = 1.005 - ((Math.log(Math.log(Math.sqrt(zz)))) / log2);
			var n1:int = Math.floor(n + p);
			var n2:int = Math.ceil(n + p);
			var p1:Number = (n + p) % n1;
			return ColorUtil.blend(color(n1), color(n2), p1);
		}
		
		
	 	/**
		 *  Returns an exact copy of the object
		 */
		public function clone():SmoothMapping
		{
			return (new SmoothMapping(_palette, _size));
		}
		
		/**
		 * Returns a representation of the object as a string
		 */
		override public function toString():String {
			return "SmoothMapping";
		}
		
    }
}
